# Author: Moon Kim 20130201
# Purpose: Dissolves each polygons to one value polygon by "GRIDCODE" 
# then adds and update fields. Then appends and projects polygons.
# runs on ArcGIS 10.0 (maybe on 10.1); read folowing submission doc for more infor.
# http://water.usgs.gov/osw/flood_inundation/toolbox/files/FloodLibraryTools/FIMProgramMapperSubmissionProcess071712.docx

# Import the arcpy.
import arcpy
from arcpy import env

# Define input values. ***MUST CHANGE THESE VALUES BEFORE RUNNING THE SCRIPT***
# Input folder 
uinWS = r"P:\indiana_inundation\lafayette\prev_model\analysis_20121029a\erased_poly"
# Output folder 
uoutWS = r"P:\indiana_inundation\lafayette\prev_model\analysis_20121029a\erased_poly\to_wim"
# Starting elevation
uinElev = 514.8
# Starting stage
uinStg = 9
# USGS staid
uinStnid = "04449999"

# Input feature classes 
arcpy.env.workspace = uinWS
inFCs = arcpy.ListFeatureClasses("*", "polygon")
# Sort list (or list stays scrambled)
inFCs.sort(reverse=False)


# Loop through each feature class and dissolve by GRIDCODE field.
for inFC in inFCs:
    # Delete polygons from previous script runs.
    if arcpy.Exists(uoutWS + "/" + inFC):
        arcpy.Delete_management(uoutWS + "/" + inFC, "POLYGON")
    print "Dissolving " + inFC
    arcpy.Dissolve_management(uinWS + "/" + inFC, uoutWS + "/" + inFC, "GRIDCODE", "", "MULTI_PART", "DISSOLVE_LINES")

# Loop through each feature class and add followng fields and fill in values based on above input values.
for inFC in inFCs:
    print "Adding field ELEV to " + inFC
    arcpy.AddField_management(uoutWS + "/" + inFC, "ELEV", "DOUBLE", "", "", "", "", "NON_NULLABLE", "NON_REQUIRED", "")
    print "Adding field STAGE to " + inFC
    arcpy.AddField_management(uoutWS + "/" + inFC, "STAGE", "DOUBLE", "", "", "", "", "NON_NULLABLE", "NON_REQUIRED", "")
    print "Adding field GRIDID to " + inFC
    arcpy.AddField_management(uoutWS + "/" + inFC, "GRIDID", "LONG", "", "", "", "", "NON_NULLABLE", "NON_REQUIRED", "")
    print "Adding field USGSID to " + inFC
    arcpy.AddField_management(uoutWS + "/" + inFC, "USGSID", "TEXT", "", "", "20", "", "NON_NULLABLE", "NON_REQUIRED", "")
    # Establish update cursor and move it to first row.
    rows = arcpy.UpdateCursor(uoutWS + "/" + inFC)
    row = rows.next()
    # Fill in values for field based on user input above.
    row.ELEV = uinElev
    row.STAGE = uinStg
    row.GRIDID = uinStg
    row.USGSID = uinStnid
    print "Updated ELEV to " + str(uinElev)
    print "Updated STAGE to " + str(uinStg)
    print "Updated GRIDID to " + str(uinStg)
    print "Updated USGSID to " + uinStnid
    # Increase by one
    uinElev = uinElev + 1
    uinStg = uinStg + 1
    rows.updateRow(row) 
    row = rows.next()
    
# Delete cursor and row objects to remove locks on the data 
# 
del row 
del rows

# Delete appended polygon from previous script runs.
if arcpy.Exists(uoutWS + "/" + "appended.shp"):
    arcpy.Delete_management(uoutWS + "/" + "appended.shp", "POLYGON")

# Create empty polygon shapefile for appending.
arcpy.CreateFeatureclass_management (uoutWS, "appended.shp", "polygon", uoutWS + "/" + inFCs[1], "", "", uoutWS + "/" + inFCs[1])

# Append all polygons in output folder
for inFC in inFCs:
    print "Appending " + inFC
    arcpy.Append_management(uoutWS + "/" + inFC, uoutWS + "/" + "appended.shp")

# Delete appended_projeced polygon from previous script runs.
if arcpy.Exists(uoutWS + "/" + "appended_projected.shp"):
    arcpy.Delete_management(uoutWS + "/" + "appended_projected.shp", "POLYGON")

# then reproject to Web Mercator WGS84
outProjFile = r"C:\ArcGIS\Desktop10.0\Coordinate Systems\Projected Coordinate Systems\World\WGS 1984 Web Mercator (Auxiliary Sphere).prj"
print "Reprojecting..." 
arcpy.Project_management(uoutWS + "/" + "appended.shp", uoutWS + "/" + "appended_projected.shp", outProjFile, "NAD_1983_to_WGS_1984_1")

# Lastly name the final shapefile appropriately (see following doc for details)
# http://water.usgs.gov/osw/flood_inundation/toolbox/files/FloodLibraryTools/FIMProgramMapperSubmissionProcess071712.docx
